@@ -1,7 +1,6 @@
-/* SPDX-License-Identifier: GPL-2.0 */
 /******************************************************************************
  *
- * Copyright(c) 2007 - 2011 Realtek Corporation. All rights reserved.
+ * Copyright(c) 2007 - 2017 Realtek Corporation.
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of version 2 of the GNU General Public License as
@@ -12,12 +11,7 @@
  * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
  * more details.
  *
- * You should have received a copy of the GNU General Public License along with
- * this program; if not, write to the Free Software Foundation, Inc.,
- * 51 Franklin Street, Fifth Floor, Boston, MA 02110, USA
- *
- *
- ******************************************************************************/
+ *****************************************************************************/
 #define _HCI_INTF_C_
 
 #include <drv_types.h>
@@ -25,22 +19,40 @@
 
 #include <linux/pci_regs.h>
 
+#ifndef CONFIG_PCI_HCI
+
+	#error "CONFIG_PCI_HCI shall be on!\n"
+
+#endif
+
+
+#if defined(PLATFORM_LINUX) && defined(PLATFORM_WINDOWS)
+
+	#error "Shall be Linux or Windows, but not both!\n"
+
+#endif
+
+#ifdef CONFIG_80211N_HT
 	extern int rtw_ht_enable;
 	extern int rtw_bw_mode;
 	extern int rtw_ampdu_enable;/* for enable tx_ampdu */
+#endif
 
+#ifdef CONFIG_GLOBAL_UI_PID
 int ui_pid[3] = {0, 0, 0};
+#endif
 
 extern int pm_netdev_open(struct net_device *pnetdev, u8 bnormal);
 int rtw_resume_process(_adapter *padapter);
 
-#ifdef CONFIG_PM_SLEEP
-static int rtw_pci_suspend(struct pci_dev *pdev, pm_message_t state);
-static int rtw_pci_resume(struct pci_dev *pdev);
+#ifdef CONFIG_PM
+	static int rtw_pci_suspend(struct pci_dev *pdev, pm_message_t state);
+	static int rtw_pci_resume(struct pci_dev *pdev);
 #endif
 
 static int rtw_drv_init(struct pci_dev *pdev, const struct pci_device_id *pdid);
 static void rtw_dev_remove(struct pci_dev *pdev);
+static void rtw_dev_shutdown(struct pci_dev *pdev);
 
 static struct specific_device_id specific_device_id_tbl[] = {
 	{.idVendor = 0x0b05, .idProduct = 0x1791, .flags = SPEC_DEV_ID_DISABLE_HT},
@@ -49,9 +61,38 @@ static struct specific_device_id specific_device_id_tbl[] = {
 };
 
 struct pci_device_id rtw_pci_id_tbl[] = {
+#ifdef CONFIG_RTL8188E
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0x8179), .driver_data = RTL8188E},
+#endif
+#ifdef CONFIG_RTL8812A
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0x8812), .driver_data = RTL8812},
+#endif
+#ifdef CONFIG_RTL8821A
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0x8821), .driver_data = RTL8821},
+#endif
+#ifdef CONFIG_RTL8192E
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0x818B), .driver_data = RTL8192E},
+#endif
+#ifdef CONFIG_RTL8192F
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xf192), .driver_data = RTL8192F},
+#endif
+#ifdef CONFIG_RTL8723B
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xb723), .driver_data = RTL8723B},
+#endif
+#ifdef CONFIG_RTL8723D
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xd723), .driver_data = RTL8723D},
+#endif
+#ifdef CONFIG_RTL8814A
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0x8813), .driver_data = RTL8814A},
+#endif
+#ifdef CONFIG_RTL8822B
+	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xB822), .driver_data = RTL8822B},
+#endif
+#ifdef CONFIG_RTL8821C
 	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xC821), .driver_data = RTL8821C},
 	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xC82A), .driver_data = RTL8821C},
 	{PCI_DEVICE(PCI_VENDER_ID_REALTEK, 0xC82B), .driver_data = RTL8821C},
+#endif
 
 	{},
 };
@@ -61,20 +102,23 @@ struct pci_drv_priv {
 	int drv_registered;
 };
 
+
 static struct pci_drv_priv pci_drvpriv = {
 	.rtw_pci_drv.name = (char *)DRV_NAME,
 	.rtw_pci_drv.probe = rtw_drv_init,
 	.rtw_pci_drv.remove = rtw_dev_remove,
-	.rtw_pci_drv.shutdown = rtw_dev_remove,
+	.rtw_pci_drv.shutdown = rtw_dev_shutdown,
 	.rtw_pci_drv.id_table = rtw_pci_id_tbl,
-#ifdef CONFIG_PM_SLEEP
+#ifdef CONFIG_PM
 	.rtw_pci_drv.suspend = rtw_pci_suspend,
 	.rtw_pci_drv.resume = rtw_pci_resume,
 #endif
 };
 
+
 MODULE_DEVICE_TABLE(pci, rtw_pci_id_tbl);
 
+
 static u16 pcibridge_vendors[PCI_BRIDGE_VENDOR_MAX] = {
 	INTEL_VENDOR_ID,
 	ATI_VENDOR_ID,
@@ -175,8 +219,19 @@ void rtw_pci_aspm_config_clkreql0sl1(_adapter *padapter)
 	else
 		tmp8 &= (~BIT7);
 
+	/* Default set L1 entrance latency to 16us */
+	/* L0s: b[0-2], L1: b[3-5]*/
+	if (pHalData->pci_backdoor_ctrl & PCI_BC_ASPM_L1) {
+		tmp8 &= (~0x38);
+		tmp8 |= 0x20;
+#ifdef CONFIG_PCI_DYNAMIC_ASPM
+		pHalData->bAspmL1LastIdle = 1;
+#endif
+	}
+
 	rtw_hal_pci_dbi_write(padapter, 0x70f, tmp8);
 
+
 	/* 0x719 Bit 3 for L1 ,  Bit4 for clock req */
 	tmp8 = rtw_hal_pci_dbi_read(padapter, 0x719);
 
@@ -231,13 +286,75 @@ void rtw_pci_aspm_config_l1off_general(_adapter *padapter, u8 enablel1off)
 
 }
 
+#ifdef CONFIG_PCI_DYNAMIC_ASPM
+void rtw_pci_aspm_config_dynamic_l1_ilde_time(_adapter *padapter)
+{
+	BOOLEAN	 bCurrentIdle = 1;	/* Default idle 4us (0x70F = 0x17)*/
+	HAL_DATA_TYPE *pHalData	= GET_HAL_DATA(padapter);
+	struct mlme_priv *pmlmepriv = &(padapter->mlmepriv);
+	struct dvobj_priv *pdvobjpriv = adapter_to_dvobj(padapter);
+	int current_tx_tp = pdvobjpriv->traffic_stat.cur_tx_tp;
+	int current_rx_tp = pdvobjpriv->traffic_stat.cur_rx_tp;
+	int current_tp = current_tx_tp + current_rx_tp;
+	u8 tmp8 = 0;
+
+	if (padapter->registrypriv.wifi_spec)
+		return;
+
+	if (!(pHalData->pci_backdoor_ctrl & PCI_BC_ASPM_L1))
+		return;
+
+#if 0
+	RTW_INFO("current_tx_tp = %d\n", current_tx_tp);
+	RTW_INFO("current_rx_tp = %d\n", current_rx_tp);
+	RTW_INFO("current_tp = %d\n", current_tp);
+#endif
+
+	if ((rtw_linked_check(padapter) == _TRUE) && 
+		((current_tx_tp >= 50)||
+		(current_rx_tp >= 50)))
+		/*(current_rx_tp >= 10))*/
+		/*(current_tp >= 10))*/
+	{
+		bCurrentIdle = 0;
+	}	
+	else
+	{
+		bCurrentIdle = 1;
+	}
+
+	if(bCurrentIdle != pHalData->bAspmL1LastIdle)
+	{
+		pHalData->bAspmL1LastIdle = bCurrentIdle;
+
+		tmp8 = rtw_hal_pci_dbi_read(padapter, 0x70F);
+		tmp8 &= (~0x38);
+
+		if(bCurrentIdle) {
+			/*tmp8 |= 0x10; *//*L1 entrance latency: 4us*/
+			/*tmp8 |= 0x18; *//*L1 entrance latency: 8us*/
+			tmp8 |= 0x20; /*L1 entrance latency: 16us*/
+			rtw_hal_pci_dbi_write(padapter, 0x70F, tmp8 );
+		}
+		else {
+			tmp8 |= 0x28; /*L1 entrance latency: 32us*/
+			rtw_hal_pci_dbi_write(padapter, 0x70F, tmp8 );
+		}
+	}
+
+}
+#endif
+
 void rtw_pci_dump_aspm_info(_adapter *padapter)
 {
+	HAL_DATA_TYPE	*pHalData = GET_HAL_DATA(padapter);
 	struct dvobj_priv	*pdvobjpriv = adapter_to_dvobj(padapter);
 	struct pci_priv	*pcipriv = &(pdvobjpriv->pcipriv);
 	u8 tmp8 = 0;
 	u16 tmp16 = 0;
 	u32 tmp32 = 0;
+	u8 l1_idle = 0;
+
 
 	RTW_INFO("***** ASPM Capability *****\n");
 
@@ -268,6 +385,7 @@ void rtw_pci_dump_aspm_info(_adapter *padapter)
 	RTW_INFO("CLK REQ:	%s\n", (tmp8 & BIT4) ? "Enable" : "Disable");
 
 	tmp8 = rtw_hal_pci_dbi_read(padapter, 0x70f);
+	l1_idle = tmp8 & 0x38;
 	RTW_INFO("ASPM L0s:	%s\n", (tmp8&BIT7) ? "Enable" : "Disable");
 
 	tmp8 = rtw_hal_pci_dbi_read(padapter, 0x719);
@@ -275,6 +393,9 @@ void rtw_pci_dump_aspm_info(_adapter *padapter)
 
 	tmp8 = rtw_hal_pci_dbi_read(padapter, 0x718);
 	RTW_INFO("ASPM L1OFF:%s\n", (tmp8 & BIT5) ? "Enable" : "Disable");
+	
+	RTW_INFO("********* MISC **********\n");
+	RTW_INFO("ASPM L1 Idel Time: 0x%x\n", l1_idle>>3);
 	RTW_INFO("*************************\n");
 }
 
@@ -291,6 +412,7 @@ static u8 rtw_pci_platform_switch_device_pci_aspm(_adapter *padapter, u8 value)
 	struct pci_priv	*pcipriv = &(pdvobjpriv->pcipriv);
 	BOOLEAN		bResult = _FALSE;
 	int	Result = 0;
+	int	error;
 
 	Result = pci_write_config_byte(pdvobjpriv->ppcidev, pcipriv->pciehdr_offset + 0x10, value);	/* enable I/O space */
 	RTW_INFO("PlatformSwitchDevicePciASPM(0x%x) = 0x%x\n", pcipriv->pciehdr_offset + 0x10, value);
@@ -339,6 +461,7 @@ void rtw_pci_disable_aspm(_adapter *padapter)
 	u8	linkctrl_reg;
 	u8	pcibridge_linkctrlreg, aspmlevel = 0;
 
+
 	/* We shall check RF Off Level for ASPM function instead of registry settings, revised by Roger, 2013.03.29. */
 	if (!(pwrpriv->reg_rfps_level & (RT_RF_LPS_LEVEL_ASPM | RT_RF_PS_LEVEL_ALWAYS_ASPM)))
 		return;
@@ -410,6 +533,7 @@ void rtw_pci_enable_aspm(_adapter *padapter)
 	u8	u_device_aspmsetting = 0;
 	u32	u_device_aspmsupportsetting = 0;
 
+
 	/* We shall check RF Off Level for ASPM function instead of registry settings, revised by Roger, 2013.03.29. */
 	if (!(pwrpriv->reg_rfps_level & (RT_RF_LPS_LEVEL_ASPM | RT_RF_PS_LEVEL_ALWAYS_ASPM)))
 		return;
@@ -598,6 +722,7 @@ _SET_CACHELINE_SIZE_FAIL:
 	return _FAIL;
 }
 
+
 #define PCI_CMD_ENABLE_BUS_MASTER		BIT(2)
 #define PCI_CMD_DISABLE_INTERRUPT		BIT(10)
 #define CMD_BUS_MASTER				BIT(2)
@@ -608,7 +733,7 @@ static s32 rtw_pci_parse_configuration(struct pci_dev *pdev, struct dvobj_priv *
 	/* PPCI_COMMON_CONFIG pPciConfig = (PPCI_COMMON_CONFIG) pucBuffer; */
 	/* u16	usPciCommand = pPciConfig->Command; */
 	u16	usPciCommand = 0;
-	int	Result, ret;
+	int	Result, ret = _FAIL;
 	u8	CapabilityOffset;
 	RT_PCI_CAPABILITIES_HEADER CapabilityHdr;
 	u8	PCIeCap;
@@ -898,8 +1023,28 @@ static void rtw_pci_initialize_adapter_common(_adapter *padapter)
 /*
  * 2009/10/28 MH Enable rtl8192ce DMA64 function. We need to enable 0x719 BIT5
  *   */
+#ifdef CONFIG_64BIT_DMA
+u8 PlatformEnableDMA64(PADAPTER Adapter)
+{
+	struct dvobj_priv	*pdvobjpriv = adapter_to_dvobj(Adapter);
+	struct pci_dev	*pdev = pdvobjpriv->ppcidev;
+	u8	bResult = _TRUE;
+	u8	value;
 
-#define rtw_pci_interrupt(x, y, z) rtw_pci_interrupt(x, y)
+	pci_read_config_byte(pdev, 0x719, &value);
+
+	/* 0x719 Bit5 is DMA64 bit fetch. */
+	value |= (BIT5);
+
+	pci_write_config_byte(pdev, 0x719, value);
+
+	return bResult;
+}
+#endif
+
+#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 0)) || (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 18))
+	#define rtw_pci_interrupt(x, y, z) rtw_pci_interrupt(x, y)
+#endif
 
 static irqreturn_t rtw_pci_interrupt(int irq, void *priv, struct pt_regs *regs)
 {
@@ -916,6 +1061,9 @@ static irqreturn_t rtw_pci_interrupt(int irq, void *priv, struct pt_regs *regs)
 	return IRQ_HANDLED;
 }
 
+#ifdef RTK_DMP_PLATFORM
+	#define pci_iounmap(x, y) iounmap(y)
+#endif
 
 int pci_alloc_irq(struct dvobj_priv *dvobj)
 {
@@ -947,10 +1095,72 @@ static void rtw_decide_chip_type_by_pci_driver_data(struct dvobj_priv *pdvobj, c
 {
 	pdvobj->chip_type = pdid->driver_data;
 
+#ifdef CONFIG_RTL8188E
+	if (pdvobj->chip_type == RTL8188E) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8188EE;
+		RTW_INFO("CHIP TYPE: RTL8188E\n");
+	}
+#endif
+
+#ifdef CONFIG_RTL8812A
+	if (pdvobj->chip_type == RTL8812) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8812E;
+		RTW_INFO("CHIP TYPE: RTL8812AE\n");
+	}
+#endif
+
+#ifdef CONFIG_RTL8821A
+	if (pdvobj->chip_type == RTL8821) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8821E;
+		RTW_INFO("CHIP TYPE: RTL8821AE\n");
+	}
+#endif
+
+#ifdef CONFIG_RTL8723B
+	if (pdvobj->chip_type == RTL8723B) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8723BE;
+		RTW_INFO("CHIP TYPE: RTL8723BE\n");
+	}
+#endif
+#ifdef CONFIG_RTL8723D
+	if (pdvobj->chip_type == RTL8723D) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8723DE;
+		RTW_INFO("CHIP TYPE: RTL8723DE\n");
+	}
+#endif
+#ifdef CONFIG_RTL8192E
+	if (pdvobj->chip_type == RTL8192E) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8192EE;
+		RTW_INFO("CHIP TYPE: RTL8192EE\n");
+	}
+#endif
+
+#ifdef CONFIG_RTL8192F
+	if (pdvobj->chip_type == RTL8192F) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8192FE;
+		RTW_INFO("CHIP TYPE: RTL8192FE\n");
+	}
+#endif
+#ifdef CONFIG_RTL8814A
+	if (pdvobj->chip_type == RTL8814A) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8814AE;
+		RTW_INFO("CHIP TYPE: RTL8814AE\n");
+	}
+#endif
+
+#if defined(CONFIG_RTL8822B)
+	if (pdvobj->chip_type == RTL8822B) {
+		pdvobj->HardwareType = HARDWARE_TYPE_RTL8822BE;
+		RTW_INFO("CHIP TYPE: RTL8822BE\n");
+	}
+#endif
+
+#if defined(CONFIG_RTL8821C)
 	if (pdvobj->chip_type == RTL8821C) {
 		pdvobj->HardwareType = HARDWARE_TYPE_RTL8821CE;
 		RTW_INFO("CHIP TYPE: RTL8821CE\n");
 	}
+#endif
 
 }
 
@@ -963,23 +1173,38 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 	struct pci_dev	*bridge_pdev = pdev->bus->self;
 	/* u32	pci_cfg_space[16]; */
 	unsigned long pmem_start, pmem_len, pmem_flags;
+	u8	tmp;
 	u8	PciBgVIdIdx;
 	int	i;
 
+
 	dvobj = devobj_init();
 	if (dvobj == NULL)
 		goto exit;
 
+
 	dvobj->ppcidev = pdev;
 	pcipriv = &(dvobj->pcipriv);
 	pci_set_drvdata(pdev, dvobj);
 
+
 	err = pci_enable_device(pdev);
 	if (err != 0) {
 		RTW_ERR("%s : Cannot enable new PCI device\n", pci_name(pdev));
 		goto free_dvobj;
 	}
 
+#ifdef CONFIG_64BIT_DMA
+	if (!pci_set_dma_mask(pdev, DMA_BIT_MASK(64))) {
+		RTW_INFO("RTL819xCE: Using 64bit DMA\n");
+		err = pci_set_consistent_dma_mask(pdev, DMA_BIT_MASK(64));
+		if (err != 0) {
+			RTW_ERR("Unable to obtain 64bit DMA for consistent allocations\n");
+			goto disable_picdev;
+		}
+		dvobj->bdma64 = _TRUE;
+	} else
+#endif
 	{
 		if (!pci_set_dma_mask(pdev, DMA_BIT_MASK(32))) {
 			err = pci_set_consistent_dma_mask(pdev, DMA_BIT_MASK(32));
@@ -988,6 +1213,7 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 				goto disable_picdev;
 			}
 		}
+		dvobj->bdma64 = _FALSE;
 	}
 
 	pci_set_master(pdev);
@@ -998,6 +1224,22 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 		goto disable_picdev;
 	}
 
+#ifdef RTK_129X_PLATFORM
+	if (pdev->bus->number == 0x00) {
+		pmem_start = PCIE_SLOT1_MEM_START;
+		pmem_len   = PCIE_SLOT1_MEM_LEN;
+		pmem_flags = 0;
+		RTW_PRINT("RTD129X: PCIE SLOT1\n");
+	} else if (pdev->bus->number == 0x01) {
+		pmem_start = PCIE_SLOT2_MEM_START;
+		pmem_len   = PCIE_SLOT2_MEM_LEN;
+		pmem_flags = 0;
+		RTW_PRINT("RTD129X: PCIE SLOT2\n");
+	} else {
+		RTW_ERR(KERN_ERR "RTD129X: Wrong Slot Num\n");
+		goto release_regions;
+	}
+#else
 	/* Search for memory map resource (index 0~5) */
 	for (i = 0 ; i < 6 ; i++) {
 		pmem_start = pci_resource_start(pdev, i);
@@ -1012,9 +1254,32 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 		RTW_ERR("%s: No MMIO resource found, abort!\n", __func__);
 		goto release_regions;
 	}
+#endif /* RTK_DMP_PLATFORM */
 
+#ifdef RTK_DMP_PLATFORM
+	dvobj->pci_mem_start = (unsigned long)ioremap_nocache(pmem_start, pmem_len);
+#elif defined(RTK_129X_PLATFORM)
+	if (pdev->bus->number == 0x00)
+		dvobj->ctrl_start =
+			(unsigned long)ioremap(PCIE_SLOT1_CTRL_START, 0x200);
+	else if (pdev->bus->number == 0x01)
+		dvobj->ctrl_start =
+			(unsigned long)ioremap(PCIE_SLOT2_CTRL_START, 0x200);
+
+	if (dvobj->ctrl_start == 0) {
+		RTW_ERR("RTD129X: Can't map CTRL mem\n");
+		goto release_regions;
+	}
+
+	dvobj->mask_addr = dvobj->ctrl_start + PCIE_MASK_OFFSET;
+	dvobj->tran_addr = dvobj->ctrl_start + PCIE_TRANSLATE_OFFSET;
+
+	dvobj->pci_mem_start =
+		(unsigned long)ioremap_nocache(pmem_start, pmem_len);
+#else
 	/* shared mem start */
 	dvobj->pci_mem_start = (unsigned long)pci_iomap(pdev, i, pmem_len);
+#endif
 	if (dvobj->pci_mem_start == 0) {
 		RTW_ERR("Can't map PCI mem\n");
 		goto release_regions;
@@ -1038,10 +1303,17 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 		pcipriv->pcibridge_deviceid = bridge_pdev->device;
 	}
 
+#if 0
+	/* Read PCI configuration Space Header */
+	for (i = 0; i < 16; i++)
+		pci_read_config_dword(pdev, (i << 2), &pci_cfg_space[i]);
+#endif
+
 	/*step 1-1., decide the chip_type via device info*/
 	dvobj->interface_type = RTW_PCIE;
 	rtw_decide_chip_type_by_pci_driver_data(dvobj, pdid);
 
+
 	/* rtw_pci_parse_configuration(pdev, dvobj, (u8 *)&pci_cfg_space); */
 	rtw_pci_parse_configuration(pdev, dvobj);
 
@@ -1062,11 +1334,21 @@ static struct dvobj_priv	*pci_dvobj_init(struct pci_dev *pdev, const struct pci_
 
 	status = _SUCCESS;
 
+iounmap:
 	if (status != _SUCCESS && dvobj->pci_mem_start != 0) {
+#if 1/* def RTK_DMP_PLATFORM */
 		pci_iounmap(pdev, (void *)dvobj->pci_mem_start);
+#endif
 		dvobj->pci_mem_start = 0;
 	}
 
+#ifdef RTK_129X_PLATFORM
+	if (status != _SUCCESS && dvobj->ctrl_start != 0) {
+		pci_iounmap(pdev, (void *)dvobj->ctrl_start);
+		dvobj->ctrl_start = 0;
+	}
+#endif
+
 release_regions:
 	if (status != _SUCCESS)
 		pci_release_regions(pdev);
@@ -1083,6 +1365,7 @@ exit:
 	return dvobj;
 }
 
+
 static void pci_dvobj_deinit(struct pci_dev *pdev)
 {
 	struct dvobj_priv *dvobj = pci_get_drvdata(pdev);
@@ -1096,10 +1379,18 @@ static void pci_dvobj_deinit(struct pci_dev *pdev)
 		}
 
 		if (dvobj->pci_mem_start != 0) {
+#if 1/* def RTK_DMP_PLATFORM */
 			pci_iounmap(pdev, (void *)dvobj->pci_mem_start);
+#endif
 			dvobj->pci_mem_start = 0;
 		}
 
+#ifdef RTK_129X_PLATFORM
+		if (dvobj->ctrl_start != 0) {
+			pci_iounmap(pdev, (void *)dvobj->ctrl_start);
+			dvobj->ctrl_start = 0;
+		}
+#endif
 		devobj_deinit(dvobj);
 	}
 
@@ -1108,14 +1399,56 @@ static void pci_dvobj_deinit(struct pci_dev *pdev)
 
 }
 
+
 u8 rtw_set_hal_ops(_adapter *padapter)
 {
 	/* alloc memory for HAL DATA */
 	if (rtw_hal_data_init(padapter) == _FAIL)
 		return _FAIL;
 
+#ifdef CONFIG_RTL8188E
+	if (rtw_get_chip_type(padapter) == RTL8188E)
+		rtl8188ee_set_hal_ops(padapter);
+#endif
+
+#if defined(CONFIG_RTL8812A) || defined(CONFIG_RTL8821A)
+	if ((rtw_get_chip_type(padapter) == RTL8812) || (rtw_get_chip_type(padapter) == RTL8821))
+		rtl8812ae_set_hal_ops(padapter);
+#endif
+
+#ifdef CONFIG_RTL8723B
+	if (rtw_get_chip_type(padapter) == RTL8723B)
+		rtl8723be_set_hal_ops(padapter);
+#endif
+
+#ifdef CONFIG_RTL8723D
+	if (rtw_get_chip_type(padapter) == RTL8723D)
+		rtl8723de_set_hal_ops(padapter);
+#endif
+
+#ifdef CONFIG_RTL8192E
+	if (rtw_get_chip_type(padapter) == RTL8192E)
+		rtl8192ee_set_hal_ops(padapter);
+#endif
+
+#ifdef CONFIG_RTL8192F
+	if (rtw_get_chip_type(padapter) == RTL8192F)
+		rtl8192fe_set_hal_ops(padapter);
+#endif
+
+#ifdef CONFIG_RTL8814A
+	if (rtw_get_chip_type(padapter) == RTL8814A)
+		rtl8814ae_set_hal_ops(padapter);
+#endif
+
+#if defined(CONFIG_RTL8822B)
+	if (rtw_get_chip_type(padapter) == RTL8822B)
+		rtl8822be_set_hal_ops(padapter);
+#endif
+#if defined(CONFIG_RTL8821C)
 	if (rtw_get_chip_type(padapter) == RTL8821C)
 		rtl8821ce_set_hal_ops(padapter);
+#endif
 
 	if (rtw_hal_ops_check(padapter) == _FAIL)
 		return _FAIL;
@@ -1128,9 +1461,50 @@ u8 rtw_set_hal_ops(_adapter *padapter)
 
 void pci_set_intf_ops(_adapter *padapter, struct _io_ops *pops)
 {
+#ifdef CONFIG_RTL8188E
+	if (rtw_get_chip_type(padapter) == RTL8188E)
+		rtl8188ee_set_intf_ops(pops);
+#endif
 
+#if defined(CONFIG_RTL8812A) || defined(CONFIG_RTL8821A)
+	if ((rtw_get_chip_type(padapter) == RTL8812) || (rtw_get_chip_type(padapter) == RTL8821))
+		rtl8812ae_set_intf_ops(pops);
+#endif
+
+#ifdef CONFIG_RTL8723B
+	if (rtw_get_chip_type(padapter) == RTL8723B)
+		rtl8723be_set_intf_ops(pops);
+#endif
+
+#ifdef CONFIG_RTL8723D
+	if (rtw_get_chip_type(padapter) == RTL8723D)
+		rtl8723de_set_intf_ops(pops);
+#endif
+
+#ifdef CONFIG_RTL8192E
+	if (rtw_get_chip_type(padapter) == RTL8192E)
+		rtl8192ee_set_intf_ops(pops);
+#endif
+
+#ifdef CONFIG_RTL8192F
+	if (rtw_get_chip_type(padapter) == RTL8192F)
+		rtl8192fe_set_intf_ops(pops);
+#endif
+
+#ifdef CONFIG_RTL8814A
+	if (rtw_get_chip_type(padapter) == RTL8814A)
+		rtl8814ae_set_intf_ops(pops);
+#endif
+
+#if defined(CONFIG_RTL8822B)
+	if (rtw_get_chip_type(padapter) == RTL8822B)
+		rtl8822be_set_intf_ops(pops);
+#endif
+
+#if defined(CONFIG_RTL8821C)
 	if (rtw_get_chip_type(padapter) == RTL8821C)
 		rtl8821ce_set_intf_ops(pops);
+#endif
 
 }
 
@@ -1163,6 +1537,7 @@ static void rtw_mi_pci_tasklets_kill(_adapter *padapter)
 static void pci_intf_stop(_adapter *padapter)
 {
 
+
 	/* Disable hw interrupt */
 	if (!rtw_is_surprise_removed(padapter)) {
 		/* device still exists, so driver can do i/o operation */
@@ -1178,10 +1553,12 @@ static void pci_intf_stop(_adapter *padapter)
 		adapter_to_dvobj(padapter)->irq_enabled = 0;
 	}
 
+
 }
 
 static void disable_ht_for_spec_devid(const struct pci_device_id *pdid)
 {
+#ifdef CONFIG_80211N_HT
 	u16 vid, pid;
 	u32 flags;
 	int i;
@@ -1199,9 +1576,10 @@ static void disable_ht_for_spec_devid(const struct pci_device_id *pdid)
 		}
 
 	}
+#endif
 }
 
-#ifdef CONFIG_PM_SLEEP
+#ifdef CONFIG_PM
 static int rtw_pci_suspend(struct pci_dev *pdev, pm_message_t state)
 {
 	int ret = 0;
@@ -1215,8 +1593,16 @@ static int rtw_pci_suspend(struct pci_dev *pdev, pm_message_t state)
 		goto exit;
 	}
 
+#ifdef CONFIG_WOWLAN
+	device_set_wakeup_enable(&pdev->dev, true);
+#endif
 	pci_disable_device(pdev);
 
+#ifdef CONFIG_WOWLAN
+	ret = pci_enable_wake(pdev, pci_choose_state(pdev, state), true);
+	if (ret != 0)
+		RTW_INFO("%s Failed on pci_enable_wake (%d)\n", __func__, ret);
+#endif
 	ret = pci_set_power_state(pdev, pci_choose_state(pdev, state));
 	if (ret != 0)
 		RTW_INFO("%s Failed on pci_set_power_state (%d)\n", __func__, ret);
@@ -1235,6 +1621,7 @@ static int rtw_pci_resume(struct pci_dev *pdev)
 {
 	struct dvobj_priv *dvobj = pci_get_drvdata(pdev);
 	_adapter *padapter = dvobj_get_primary_adapter(dvobj);
+	struct net_device *pnetdev = padapter->pnetdev;
 	struct pwrctrl_priv *pwrpriv = dvobj_to_pwrctl(dvobj);
 	int	err = 0;
 
@@ -1250,32 +1637,52 @@ static int rtw_pci_resume(struct pci_dev *pdev)
 		goto exit;
 	}
 
-	pci_restore_state(pdev);
 
-	if (pwrpriv->bInternalAutoSuspend)
+#ifdef CONFIG_WOWLAN
+	err =  pci_enable_wake(pdev, PCI_D0, 0);
+	if (err != 0) {
+		RTW_INFO("%s Failed on pci_enable_wake (%d)\n", __func__, err);
+		goto exit;
+	}
+#endif
+#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 37))
+	pci_restore_state(pdev);
+#else
+	err = pci_restore_state(pdev);
+	if (err != 0) {
+		RTW_INFO("%s Failed on pci_restore_state (%d)\n", __func__, err);
+		goto exit;
+	}
+#endif
+
+#ifdef CONFIG_WOWLAN
+	device_set_wakeup_enable(&pdev->dev, false);
+#endif
+
+	if (pwrpriv->wowlan_mode || pwrpriv->wowlan_ap_mode) {
+		rtw_resume_lock_suspend();
 		err = rtw_resume_process(padapter);
-	else {
-		if (pwrpriv->wowlan_mode || pwrpriv->wowlan_ap_mode) {
+		rtw_resume_unlock_suspend();
+	} else {
+#ifdef CONFIG_RESUME_IN_WORKQUEUE
+		rtw_resume_in_workqueue(pwrpriv);
+#else
+		if (rtw_is_earlysuspend_registered(pwrpriv)) {
+			/* jeff: bypass resume here, do in late_resume */
+			rtw_set_do_late_resume(pwrpriv, _TRUE);
+		} else {
 			rtw_resume_lock_suspend();
 			err = rtw_resume_process(padapter);
 			rtw_resume_unlock_suspend();
-		} else {
-			if (rtw_is_earlysuspend_registered(pwrpriv)) {
-				/* jeff: bypass resume here, do in late_resume */
-				rtw_set_do_late_resume(pwrpriv, _TRUE);
-			} else {
-				rtw_resume_lock_suspend();
-				err = rtw_resume_process(padapter);
-				rtw_resume_unlock_suspend();
-			}
 		}
+#endif
 	}
 
 exit:
 
 	return err;
 }
-#endif /* CONFIG_PM_SLEEP */
+#endif/* CONFIG_PM */
 
 _adapter *rtw_pci_primary_adapter_init(struct dvobj_priv *dvobj, struct pci_dev *pdev)
 {
@@ -1299,7 +1706,11 @@ _adapter *rtw_pci_primary_adapter_init(struct dvobj_priv *dvobj, struct pci_dev
 	/* set adapter_type/iface type for primary padapter */
 	padapter->isprimary = _TRUE;
 	padapter->adapter_type = PRIMARY_ADAPTER;
+#ifdef CONFIG_MI_WITH_MBSSID_CAM
 	padapter->hw_port = HW_PORT0;
+#else
+	padapter->hw_port = HW_PORT0;
+#endif
 
 	if (rtw_init_io_priv(padapter, pci_set_intf_ops) == _FAIL)
 		goto free_adapter;
@@ -1319,6 +1730,11 @@ _adapter *rtw_pci_primary_adapter_init(struct dvobj_priv *dvobj, struct pci_dev
 	/* .4 */
 	rtw_hal_chip_configure(padapter);
 
+#ifdef CONFIG_BT_COEXIST
+	rtw_btcoex_Initialize(padapter);
+#endif
+	rtw_btcoex_wifionly_initialize(padapter);
+
 	/* step 4. read efuse/eeprom data and get mac_addr */
 	if (rtw_hal_read_chip_info(padapter) == _FAIL)
 		goto free_hal_data;
@@ -1327,17 +1743,17 @@ _adapter *rtw_pci_primary_adapter_init(struct dvobj_priv *dvobj, struct pci_dev
 	if (rtw_init_drv_sw(padapter) == _FAIL)
 		goto free_hal_data;
 
-	if (GET_HAL_DATA(padapter)->EEPROMBluetoothCoexist)
-		rtw_btcoex_Initialize(padapter);
-	else
-		rtw_btcoex_wifionly_initialize(padapter);
-
 	if (rtw_hal_inirp_init(padapter) == _FAIL)
 		goto free_hal_data;
 
 	rtw_macaddr_cfg(adapter_mac_addr(padapter),  get_hal_mac_addr(padapter));
 
+#ifdef CONFIG_MI_WITH_MBSSID_CAM
+	rtw_mbid_camid_alloc(padapter, adapter_mac_addr(padapter));
+#endif
+#ifdef CONFIG_P2P
 	rtw_init_wifidirect_addrs(padapter, adapter_mac_addr(padapter), adapter_mac_addr(padapter));
+#endif /* CONFIG_P2P */
 
 	rtw_hal_disable_interrupt(padapter);
 
@@ -1359,6 +1775,9 @@ free_hal_data:
 
 free_adapter:
 	if (status != _SUCCESS && padapter) {
+		#ifdef RTW_HALMAC
+		rtw_halmac_deinit_adapter(dvobj);
+		#endif
 		rtw_vmfree((u8 *)padapter, sizeof(*padapter));
 		padapter = NULL;
 	}
@@ -1375,11 +1794,19 @@ static void rtw_pci_primary_adapter_deinit(_adapter *padapter)
 	if (check_fwstate(pmlmepriv, _FW_LINKED))
 		rtw_disassoc_cmd(padapter, 0, RTW_CMDF_DIRECTLY);
 
-	if (check_fwstate(&padapter->mlmepriv, WIFI_AP_STATE) == _TRUE) {
+#ifdef CONFIG_AP_MODE
+	if (MLME_IS_AP(padapter) || MLME_IS_MESH(padapter)) {
 		free_mlme_ap_info(padapter);
+#ifdef CONFIG_HOSTAPD_MLME
+		hostapd_mode_unload(padapter);
+#endif
 	}
+#endif
 
 	/*rtw_cancel_all_timer(padapte);*/
+#ifdef CONFIG_WOWLAN
+	adapter_to_pwrctl(padapter)->wowlan_mode = _FALSE;
+#endif /* CONFIG_WOWLAN */
 	rtw_dev_unload(padapter);
 
 	RTW_INFO("%s, hw_init_completed=%s\n", __func__, rtw_is_hw_init_completed(padapter) ? "_TRUE" : "_FALSE");
@@ -1390,7 +1817,9 @@ static void rtw_pci_primary_adapter_deinit(_adapter *padapter)
 	/* TODO: use rtw_os_ndevs_deinit instead at the first stage of driver's dev deinit function */
 	rtw_os_ndev_free(padapter);
 
+#ifdef RTW_HALMAC
 	rtw_halmac_deinit_adapter(adapter_to_dvobj(padapter));
+#endif /* RTW_HALMAC */
 
 	rtw_vmfree((u8 *)padapter, sizeof(_adapter));
 
@@ -1408,9 +1837,14 @@ static void rtw_pci_primary_adapter_deinit(_adapter *padapter)
 */
 static int rtw_drv_init(struct pci_dev *pdev, const struct pci_device_id *pdid)
 {
+	int i, err = -ENODEV;
+
 	int status = _FAIL;
 	_adapter *padapter = NULL;
 	struct dvobj_priv *dvobj;
+	struct net_device *pnetdev;
+
+	/* RTW_INFO("+rtw_drv_init\n"); */
 
 	/* step 0. */
 	disable_ht_for_spec_devid(pdid);
@@ -1428,16 +1862,32 @@ static int rtw_drv_init(struct pci_dev *pdev, const struct pci_device_id *pdid)
 	}
 
 	/* Initialize virtual interface */
+#ifdef CONFIG_CONCURRENT_MODE
+	if (padapter->registrypriv.virtual_iface_num > (CONFIG_IFACE_NUMBER - 1))
+		padapter->registrypriv.virtual_iface_num = (CONFIG_IFACE_NUMBER - 1);
 
+	for (i = 0; i < padapter->registrypriv.virtual_iface_num; i++) {
+		if (rtw_drv_add_vir_if(padapter, pci_set_intf_ops) == NULL) {
+			RTW_INFO("rtw_drv_add_iface failed! (%d)\n", i);
+			goto free_if_vir;
+		}
+	}
+#endif
+
+#ifdef CONFIG_GLOBAL_UI_PID
 	if (ui_pid[1] != 0) {
 		RTW_INFO("ui_pid[1]:%d\n", ui_pid[1]);
 		rtw_signal_process(ui_pid[1], SIGUSR2);
 	}
+#endif
 
 	/* dev_alloc_name && register_netdev */
 	if (rtw_os_ndevs_init(dvobj) != _SUCCESS)
 		goto free_if_vir;
 
+#ifdef CONFIG_HOSTAPD_MLME
+	hostapd_mode_init(padapter);
+#endif
 
 #ifdef CONFIG_PLATFORM_RTD2880B
 	RTW_INFO("wlan link up\n");
@@ -1457,6 +1907,10 @@ os_ndevs_deinit:
 		rtw_os_ndevs_deinit(dvobj);
 free_if_vir:
 	if (status != _SUCCESS) {
+#ifdef CONFIG_CONCURRENT_MODE
+		rtw_drv_stop_vir_ifaces(dvobj);
+		rtw_drv_free_vir_ifaces(dvobj);
+#endif
 	}
 
 	if (status != _SUCCESS && padapter)
@@ -1477,6 +1931,7 @@ static void rtw_dev_remove(struct pci_dev *pdev)
 {
 	struct dvobj_priv *pdvobjpriv = pci_get_drvdata(pdev);
 	_adapter *padapter = dvobj_get_primary_adapter(pdvobjpriv);
+	struct net_device *pnetdev = padapter->pnetdev;
 
 	if (pdvobjpriv->processing_dev_remove == _TRUE) {
 		RTW_WARN("%s-line%d: Warning! device has been removed!\n", __func__, __LINE__);
@@ -1493,8 +1948,29 @@ static void rtw_dev_remove(struct pci_dev *pdev)
 	/* TODO: use rtw_os_ndevs_deinit instead at the first stage of driver's dev deinit function */
 	rtw_os_ndevs_unregister(pdvobjpriv);
 
+#if 0
+#ifdef RTK_DMP_PLATFORM
+	rtw_clr_surprise_removed(padapter);	/* always trate as device exists*/
+	/* this will let the driver to disable it's interrupt */
+#else
+	if (pci_drvpriv.drv_registered == _TRUE) {
+		/* RTW_INFO("r871xu_dev_remove():padapter->bSurpriseRemoved == _TRUE\n"); */
+		rtw_set_surprise_removed(padapter);
+	}
+	/*else
+	{
 
-	if (padapter->bFWReady == _TRUE) {
+		rtw_set_hw_init_completed(padapter, _FALSE);
+
+	}*/
+#endif
+#endif
+
+#if defined(CONFIG_HAS_EARLYSUSPEND) || defined(CONFIG_ANDROID_POWER)
+	rtw_unregister_early_suspend(dvobj_to_pwrctl(pdvobjpriv));
+#endif
+
+	if (GET_HAL_DATA(padapter)->bFWReady == _TRUE) {
 		rtw_pm_set_ips(padapter, IPS_NONE);
 		rtw_pm_set_lps(padapter, PS_MODE_ACTIVE);
 
@@ -1503,11 +1979,23 @@ static void rtw_dev_remove(struct pci_dev *pdev)
 
 	rtw_set_drv_stopped(padapter);	/*for stop thread*/
 	rtw_stop_cmd_thread(padapter);
+#ifdef CONFIG_CONCURRENT_MODE
+	rtw_drv_stop_vir_ifaces(pdvobjpriv);
+#endif
 
+#ifdef CONFIG_BT_COEXIST
+#ifdef CONFIG_BT_COEXIST_SOCKET_TRX
+	if (GET_HAL_DATA(padapter)->EEPROMBluetoothCoexist)
+		rtw_btcoex_close_socket(padapter);
+#endif
 	rtw_btcoex_HaltNotify(padapter);
+#endif
 
 	rtw_pci_primary_adapter_deinit(padapter);
 
+#ifdef CONFIG_CONCURRENT_MODE
+	rtw_drv_free_vir_ifaces(pdvobjpriv);
+#endif
 
 	pci_dvobj_deinit(pdev);
 
@@ -1516,6 +2004,24 @@ static void rtw_dev_remove(struct pci_dev *pdev)
 	return;
 }
 
+static void rtw_dev_shutdown(struct pci_dev *pdev)
+{
+	struct dvobj_priv *pdvobjpriv = pci_get_drvdata(pdev);
+	_adapter *padapter = dvobj_get_primary_adapter(pdvobjpriv);
+	struct net_device *pnetdev = padapter->pnetdev;
+
+#ifdef CONFIG_RTL8723D
+	if (IS_HARDWARE_TYPE_8723DE(padapter)) {
+		u1Byte u1Tmp;
+
+		u1Tmp = PlatformEFIORead1Byte(padapter, 0x75 /*REG_HCI_OPT_CTRL_8723D+1*/);
+		PlatformEFIOWrite1Byte(padapter, 0x75 /*REG_HCI_OPT_CTRL_8723D+1*/, (u1Tmp|BIT0));/*Disable USB Suspend Signal*/
+	}
+#endif
+
+	rtw_dev_remove(pdev);
+}
+
 static int __init rtw_drv_entry(void)
 {
 	int ret = 0;
@@ -1526,16 +2032,24 @@ static int __init rtw_drv_entry(void)
 	RTW_PRINT(DRV_NAME" BT-Coex version = %s\n", BTCOEXVERSION);
 #endif /* BTCOEXVERSION */
 
+#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 24))
+	/* console_suspend_enabled=0; */
+#endif
+
 	pci_drvpriv.drv_registered = _TRUE;
 	rtw_suspend_lock_init();
+	rtw_drv_proc_init();
 	rtw_ndev_notifier_register();
+	rtw_inetaddr_notifier_register();
 
 	ret = pci_register_driver(&pci_drvpriv.rtw_pci_drv);
 
 	if (ret != 0) {
 		pci_drvpriv.drv_registered = _FALSE;
 		rtw_suspend_lock_uninit();
+		rtw_drv_proc_deinit();
 		rtw_ndev_notifier_unregister();
+		rtw_inetaddr_notifier_unregister();
 		goto exit;
 	}
 
@@ -1553,12 +2067,15 @@ static void __exit rtw_drv_halt(void)
 	pci_unregister_driver(&pci_drvpriv.rtw_pci_drv);
 
 	rtw_suspend_lock_uninit();
+	rtw_drv_proc_deinit();
 	rtw_ndev_notifier_unregister();
+	rtw_inetaddr_notifier_unregister();
 
 	RTW_PRINT("module exit success\n");
 
 	rtw_mstat_dump(RTW_DBGDUMP);
 }
 
+
 module_init(rtw_drv_entry);
 module_exit(rtw_drv_halt);
